<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 10/11/2003 *
 * ===============                              Last change: 10/11/2003 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : admins_functions.php                             *
 * -------------------------------------------------------------------- *
 * Short description : Functions for the admins extension               *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer die admins-Erweiterung           *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (ereg(basename(__FILE__), $_SERVER['PHP_SELF']))
{
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}
// Caching class
class mxchange_cache
{
	// Define variables
	var $update_interval = 0;
	var $ret = "init";
	var $cache_path = "";
	var $cache_inc = "";
	var $cache_ctime = 0;
	var $cache_pointer = false;

	// Constructor
	function mxchange_cache($interval, $path, $tested) {
		// Remember interval in class
		$this->update_interval=$interval;

		// Remeber path
		$this->cache_path=$path;

		// Check if path exists
		if ((file_exists($path)) && (is_dir($path)) && (!$tested))
		{
			// Check if we can create a file inside the path
			@touch($path."dummy.tmp", 'w');
			if (file_exists($path."dummy.tmp")) {
				// Yes, we can do. So let's remove it
				@unlink($path."dummy.tmp");

				// Is there a .htaccess file?
				if (file_exists($path.".htaccess")) {
					// Update database that we have tested it
					$result = SQL_QUERY("UPDATE "._MYSQL_PREFIX."_config SET cache_tested='1' WHERE config=0 LIMIT 1", __FILE__, __LINE__);
					$this->ret="done";

					// All done!
					return "done";
				} else {
					// Stop! Set a .htaccess file first
					$this->ret="htaccess";
					return "htaccess";
				}
			}
		} elseif ($tested) {
			// System already tested
			$this->ret="done";
			return "done";
		}

		// Something goes wrong here!
		$this->ret="failed";
		return "failed";
	}

	function cache_file($file, $ignore_ctime=false)
	{
		global $INC;
		// Construct FQFN (full qualified file name)
		$inc = $this->cache_path.$file.".cache";

		// Rember it + filename in class
		$this->cache_inc = $inc;

		// Check if file exists
		$status = (file_exists($inc) && (is_readable($inc)) && (is_writeable($inc)));
		if ($status)
		{
			// Yes, it does. So let's get it's last changed date/time
			$ctime = filectime($inc);
		}
		else
		{
			// No, it doesn't. Zero date/time
			$ctime = "0";
		}

		// Remember change date/time in class
		$this->cache_ctime = $ctime;

		// Is the cache file outdated?
		if (((time() - $ctime) >= $this->update_interval) && (!$ignore_ctime))
		{
			// Ok, we need an update!
			$status = false;
		}
		return $status;
	}

	function cache_init($array)
	{
		// This will destory an existing cache file!
		if ($this->ret == "done")
		{
			// Create file
			if (file_exists($this->cache_inc)) @chmod($this->cache_inc, 0666);
			$fp = @fopen($this->cache_inc, 'w') or mxchange_die("Cannot write to cache ".$this->cache_inc." !");

			// Begin of cache file
			fwrite($fp, "\$ARRAY = \"".$array."\";\n\n");

			// Remember file pointer
			$this->cache_pointer = $fp;
		}
		else
		{
			// Cannot create file
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function add_row($data) {
		if (is_resource($this->cache_pointer)) {
			// Write every array element to cache file
			foreach ($data as $k=>$v) {
				@fwrite($this->cache_pointer, "\$data['".$k."'][] = \"".$v."\";\n");
			}
		} else {
			// Cannot create file
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_close()
	{
		// Quit function when no pointer is set
		if (empty($this->cache_pointer)) return;
		if ($this->cache_pointer)
		{
			// Close file add destroy handler
			@fclose($this->cache_pointer);

			// Set rights
			if (file_exists($this->cache_inc)) @chmod($this->cache_inc, 0666);

			// Remove pointer
			unset($this->cache_pointer);
		}
		else
		{
			// Cannot create file
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_load()
	{
		if ((file_exists($this->cache_inc)) && (is_readable($this->cache_inc)))
		{
			// Prepare temporary array
			$data = array();

			// Load cache file
			$cache = implode("", file($this->cache_inc));

			// Execute cache file
			eval($cache);
			if (is_array($data)) {
				// Return cache
				return $data;
			} else {
				// Cache problem detected!
				$this->cache_destroy();
			}
		}
		else
		{
			// Cache file not found or not readable
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_CANNOT_LOAD_1.$this->cache_inc.CACHE_CANNOT_LOAD_2);
		}
	}

	function cache_destroy()
	{
		if (file_exists($this->cache_inc))
		{
			// Remove cache file from system
			@unlink($this->cache_inc);
			if (!file_exists($this->cache_inc))
			{
				// Close cache automatically (we don't need it anymore!)
				$this->cache_close();
			}
			else
			{
				// Not removed!
				ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_CANNOT_UNLINK_1.$this->cache_inc.CACHE_CANNOT_UNLINK_2);
			}
		}
		else
		{
			// Does not exist!
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_CANNOT_UNLINK_1.$this->cache_inc.CACHE_CANNOT_UNLINK_2);
		}
	}

	function cache_remove($search, $data, $array)
	{
		global $ARRAY;
		if ((file_exists($this->cache_inc)) && (is_writeable($this->cache_inc)))
		{
			// Load cache into dummy array
			$dummy = $this->cache_load();

			// Search for key in array
			$key = array_search($data, $dummy[$search]);
			if (!empty($key))
			{
				// Key (hopefully) found?
				foreach ($array as $a)
				{
					// So we can remove all elements as requested
					unset($dummy[$a][$key]);
				}

				// Flush array to cache file
				$fp = fopen($this->cache_inc, 'w');
				fwrite($fp, "\$ARRAY = \"".$ARRAY."\";\n");
				foreach ($dummy as $k=>$v)
				{
					if (is_array($v))
					{
						// Multi line(s) found
						$LINE = "";
						foreach($v as $k2=>$v2)
						{
							// Put every array element in a row...
							$LINE .= "\$cacheInstance['".$k."'][] = \"".$v2."\";\n";
						}
					}
					else
					{
						// Single line found
						$LINE = "\$cacheInstance['".$k."'] = \"".$v."\";\n";
					}

					// Write line(s)
					fwrite($fp, $LINE);
				}

				// Close cache file
				fclose($fp);
			}
		}
		else
		{
			// Cannot write to cache!
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_PROBLEMS_DETECTED);
		}
	}

	function cache_replace($search, $replace, $search_key, $array)
	{
		global $ARRAY;
		if ((file_exists($this->cache_inc)) && (is_writeable($this->cache_inc)))
		{
			// Load cache into dummy array
			$dummy = $this->cache_load();

			// Check if $dummy is valid (prevents some errors)
			if ((is_array($dummy)) && (isset($dummy[$search])) && (is_array($dummy[$search])))
			{
				// Search for key in array
				$key_found = array_key_exists($search_key, $dummy[$search]);
				if ($key_found == true)
				{
					$key = $search_key;
					// Key (hopefully) found?
					foreach ($dummy as $a=>$v)
					{
						// So we can update all entries
						if ($a == $search)
						{
							// Update now...
							$dummy[$a][$search_key] = $replace;
						}
					}

					// Flush array to cache file
					$fp = fopen($this->cache_inc, 'w');
					fwrite($fp, "\$dummy = \"".$ARRAY."\";\n");
					foreach ($dummy as $k=>$v)
					{
						if (is_array($v))
						{
							// Multi line(s) found
							$LINE = "";
							foreach($v as $k2=>$v2)
							{
								// Put every array element in a row...
								$LINE .= "\$cacheInstance['".$k."'][] = \"".$v2."\";\n";
							}
						}
						else
						{
							// Single line found
							$LINE = "\$cacheInstance['".$k."'] = \"".$v."\";\n";
						}

						// Write line(s)
						fwrite($fp, $LINE);
					}

					// Close cache file
					fclose($fp);
				}
			}
		}
		else
		{
			// Cannot write to cache!
			ADD_FATAL(__FILE__."(".__LINE__."): ".CACHE_PROBLEMS_DETECTED);
		}
	}
}
//
?>
